/*
 * 	ACQ_TCP_FCT_LIGHT.h
 *
 *	David OPDEBECK -- Mai 1998
 *
 *      Fonction : Prototypes des fonctions utilisees pour les acces 
 *	   TCP/IP de l'acquisition.
 *
 *      Modifications :
 *        - 7 mai 98 B. Raine
 *            Passage sous Lynx : 
 *               GEN_TYPE.H => gen_type.h
 */   
 


#ifndef __ACQ_TCP_RDWR_H
#define __ACQ_TCP_RDWR_H  

#include "acq_tcp_general.h"
#include "gen_type.h"



typedef enum FONCTION_PIPE { synchro, init, nbfils, retirefils, ajoutefils,
				index_creation, nosocket, close_pipe }
			fonction_pipe;
typedef enum ACTION_PIPE { lecture, ecriture } action_pipe;

 
typedef struct STRUCT_PIPE  /* Type de message utlise pour les comm inter-process */ 
{
   INT32 action;
   BYTE buffer[80];

} struct_pipe;



typedef union MESSAGE_PIPE
{
   struct_pipe message;
   BYTE octets[sizeof(struct_pipe)];
   
} message_pipe;



#define SYNCHRO 	100
#define NBRCLONE 	110
#define INDEX 		120
#define NOSOCKET 	130


/*	INT32 acq_tcp_message_pipe ( action, fonction_pipe, valeur )
 *		Cette fonction permet de lire dans un pipe le message recherche.
 *
 *	Parametres :
 *	   Entree  :  action_pipe -- defini si on effectue une lecture ou ecriture.
 *		      fonction_pipe -- type de message a retirer.
 *		      valeur -- donnee eventuelle a transmettre.
 *
 *	   Sortie  :  La fonction retourne -1 si une erreur est survenue, 0 dans le
 *		    cas d'une demande de synchronisation, si tout s'est bien passe,
 *		    ou alors le nombre de clones en cours d'execution.
 */    
INT32 acq_tcp_message_pipe ( action_pipe, fonction_pipe, INT32 );



/* 	INT32 acq_tcp_get_time ( BYTE *chaine_retour )
 *		Cette fonction permet de recuperer la date et l'heure systeme
 *	  pour l'affichage dans les fichiers log.
 *
 *	Parametres :  
 *		Entree : chaine_retour -- chaine de 80 caracteres contenant 
 *			l'info.
 *
 *		Sortie  :  La fonction retourne -1 si une erreur est survenue 
 *		 	ou sinon ACQ_OK.
 */    
INT32 acq_tcp_get_time ( BYTE * );




/* 	void acq_tcp_kill_fils ( INT32 PID ) 
 *		Cette fonction permet de detruire un fils dont le pid est 
 *	     donne en parametre.
 *	Parametres :
 *		Entree : PID - no de pid du fils a detruire..
 *
 *		Sortie  :  neant.
 */       
void acq_tcp_kill_fils ( INT32 );


/*	INT32 acq_tcp_init_client ( INT32 * Socket )
 *		Cette fonction initialise le socket client pour pointer sur
 *	     le nouveau serveur fils. Retourne un status.
 *
 *	Parametres :
 *		Entree : Socket -- Pointeur sur un descripteur de socket.
 *		Sortie : ACQ_OK ou code erreur d'acquisition.
 */
INT32 acq_tcp_init_client ( INT32 * );


/*	INT32 acq_tcp_init_serveur ( INT32 * Socket )
 *		     Cette fonction initialise le socket serveur fils lors
 * 		des appels de la fonction acq_dn_open . Retourne un status.
 *	Parametres :
 *		Entree :  Socket --  Pointeur sur un descripteur de socket.
 */
INT32 acq_tcp_init_serveur ( INT32 * );


#endif
