/*	gen_type.h
	Copyright Acquisition group, GANIL Caen, France
*/

#ifndef _GEN_TYPE_H_
#define _GEN_TYPE_H_

#ifdef PDOS /* C ALCYON pour PDOS 3.3 */
#define void char 
#endif

/* 
ABSADD		: Absolute address . Refers to a kind of pointers that point on
		  the shortest data type allowed by both hardware and language.
*/

typedef char * ABSADD;


/*			GENERAL CONSTANTS :

D_		: Displacement .
		  This constant is to be used in pointer declarations when user
	  	  builds a position independant data structure .
		  See file RELOC:HLP .

Example of use 	: 

struct S1 {
		: 
		struct S2 *ptr ;
		: 
} ;

	With this declaration, ptr is a pointer on a structure type S2 . But we
do not know wether the pointer is an absolute or a relative address .
	If, instead of the upper declaration, user writes : struct S2 D_ ptr ;
we know that the pointer value is a relative one because "D_" means displace-
ment.

	So, when you declare position independant data structures, you should 
use this declaration for the pointers .

	WARNING : THIS DECLARATION DOES NOT ASSUME POSITION INDEPENDANCE, it is
only a way to have a visual distinction between absolute addresses and displa-
cements .

	When you declare a general use C structure ( as CHAIN type structure in
file CHAIN:H for instance ) which might be used although in position dependant 
or position independant data structures, do not use "D_" to declare the poin-
ters ( if any )in this structure . Here, you should declare the pointer as an 
absolute one, and it is of the structure user responsability to handle the 
pointers value correctly .
*/

#define D_	*		/* Displacement declaration */

/* Semaphore structure definition */
typedef struct SEMAPHORE {
	char sem;	  /* Test and set byte */
	char sem_count;	  /* Counter */
	char sem_id;	  /* Processor identifier */
	char sem_tsk;	  /* Task number */
	short  sem_loop;  /* Number of NOP in the inner loop */
	short  sem_to;	  /* Time out before "taking" the semaphore */
	} SEMAPHORE;

typedef char INT8;
typedef char BYTE;
typedef short INT16;

#ifndef PDOS
#include <unistd.h> /* pour savoir si on est POSIX ou pas */
#endif

#if defined (PDOS) || defined (_PDOS) || defined (_POSIX_VERSION)
typedef long INT32;
#else
typedef int  INT32;
#endif

typedef unsigned char UNSINT8;
typedef unsigned short UNSINT16;

#if defined (PDOS) || defined (_PDOS) || defined (_POSIX_VERSION)
typedef unsigned long UNSINT32;
#else
typedef unsigned int  UNSINT32;
#endif

#ifndef NULL
#define NULL 0
#endif

#define TRUE		1	/* "true" */
#define FALSE		0	/* "false" */

#define BYTETST  	0
#define WORDTST		1
#define LONGTST         2

#define PRESENT		1
#define ABSENT		0

#define SEM_BUSY	0x80
#define SEM_FREE	0

#define YES		1
#define NO		0

#define BUSY            1
#ifdef FREE
#undef FREE
#endif
#define FREE            0

#define BIT0		1
#define BIT1		2
#define BIT2		4
#define BIT3		8
#define BIT4		16
#define BIT5		32
#define BIT6		64
#define BIT7		128

#define BREAD		0	/* lecture par byte */
#define WREAD		1	/* lecture par word */
#define LREAD		2	/* lecture par long */
#define BWRITE		3	/* ecriture par byte */
#define WWRITE		4	/* ecriture par word */
#define LWRITE		5	/* ecriture par long */

/*----- definition des events utilises dans les taches -----------------*/
#define EVTO            128	/* Evenement delay local a chaque tache   */

/*#define dprintf 	if (_tcbptr->_u1p) printf*/

/*----- definition des differentes sorties possibles -------------------*/

/*#define O_SCREEN	_tcbptr->_unt = 1*/	/* console seule	*/
/*#define O_IMPRIM	_tcbptr->_unt = 2*/	/* imprimante seule	*/
/*#define O_IMP_SCR	_tcbptr->_unt = 3*/	/* imprimante & console	*/

/*------- Definition macro chgt de port entree et sortie tache ---------*/

/*#define INP_PORT(u)        _tcbptr->_prt = (int)u*/
/*#define OUTP_PORT(u)       _tcbptr->_u1p = (int)u*/

#endif
