/**********************************************************/
/*                                                        */
/*                 Package Hermes                         */
/*                                                        */
/* ErrCfgInit.c                                           */
/*                                                        */
/* Copyright GANIL, Groupe Informatique Physique, 1999    */
/*                                                        */
/* Auteur : Frederic Saillant                             */
/*                                                        */
/* Version : v1r1 (EXOGAM)                                */
/*                                                        */
/* Date de creation : 14 mai 1999                         */
/*                                                        */
/* Derniere mise a jour : 1er juin 1999                   */
/*                                                        */
/* Fonction ErrCfgInit()                                  */
/*                                                        */
/* Initialisation de la configuration du package Hermes   */
/**********************************************************/

#define _POSIX_SOURCE /* on veut des sources POSIX */
#define _POSIX_C_SOURCE 199506L

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

#if defined(__sparc) || defined(__osf__) /* Solaris/SPARC et DigitalUnix/Alpha */
#include <sys/fcntl.h>
#else
#ifdef __Lynx__
#include <sys/file.h>
#else
#include <file.h>
#endif
#endif

#define _ErrCfgInit_

#include "HermesPrivate.h" /* fonctions privees */
#include "HermesDef.h" /* pour MAXFILEPATHLEN */

#define CFGFILE_ENV_VAR       "HERMES_CFG_FILE"     /* nom de la variable d'environnement du fichier de config */
#define SYSTEM_CFGDIR_ENV_VAR "HERMES_CFG_DIR"      /* nom de la variable d'environnement du repertoire system */
#define USER_CFGDIR_ENV_VAR   "HERMES_CFG_DIR_USER" /* nom de la variable d'environnement du repertoire user */
#define DEFAULT_CFGFILE       "Hermes.cfg"          /* fichier de config par defaut */

extern int SendToLogger;

int ErrCfgInit ( void )
{
  int ReturnCode = 0;
  char * envcfgdir; /* variable d'env repertoire du fichier de config */
  char cfgdir[MAXFILEPATHLEN]; /* repertoire du fichier de config */
  char errdir[MAXFILEPATHLEN]; /* repertoire du fichier des erreurs */
  char * envcfgfile; /* variable d'env nom du fichier de config */
  char cfgfile[MAXFILEPATHLEN]; /* nom du fichier de config */
  char cfgpath[MAXFILEPATHLEN]; /* nom complet du fichier de config */
  int len = 0;
  int fd;
  FILE * f;
  T_ClassInfo * pNewInfo = NULL;
  T_ClassInfo * p = NULL;
  char *s1, *s2;
  char * status;
  char buf[512];
  int found = 0;

  ApplicationCode = 0;

  memset((void *)cfgdir,0,MAXFILEPATHLEN);

  memset((void *)errdir,0,MAXFILEPATHLEN);

  memset((void *)cfgfile,0,MAXFILEPATHLEN);

  memset((void *)cfgpath,0,MAXFILEPATHLEN);

  pClassConfig = NULL;

  envcfgdir = getenv( SYSTEM_CFGDIR_ENV_VAR );

  envcfgfile = getenv( CFGFILE_ENV_VAR );

  if ( envcfgfile != NULL )
  {
    if ( envcfgfile[0] != 0 )
    {
      strncpy( cfgfile, envcfgfile, MAXFILEPATHLEN );
    }
    else /* si la variable d'env HERMES_CFG_FILE est vide */
    {
      strncpy( cfgfile, DEFAULT_CFGFILE, MAXFILEPATHLEN );
    }
  }
  else /* si la variable d'env HERMES_CFG_FILE n'existe pas */
  {
    strncpy( cfgfile, DEFAULT_CFGFILE, MAXFILEPATHLEN );
  }

  cfgfile[MAXFILEPATHLEN-1] = 0; /* au cas ou on aurait tronque ... */

  len = 0;

  if ( envcfgdir != NULL )
  {
    strncpy( cfgdir, envcfgdir, MAXFILEPATHLEN );

    cfgdir[MAXFILEPATHLEN-1] = 0; /* au cas ou on aurait tronque ... */

    len = strlen(cfgdir);

#ifndef __VMS
    if ( len != 0 )
    {
      if ( cfgdir[len - 1] != '/' )
      {
        if ( len < ( MAXFILEPATHLEN - 1 ) )
	{
          cfgdir[len] = '/';
          cfgdir[len + 1] = 0;
          len = len + 1;
        }
        else
	{
          ReturnCode = -1;
          printf("Hermes - ErrCfgInit() : Nom du repertoire du fichier de config system trop long !\n");
          p = NULL;
          pClassConfig = NULL;
          return( ReturnCode );
        }
      }
    }
#endif

    strcpy( cfgpath, cfgdir );

    strncat( cfgpath, cfgfile, MAXFILEPATHLEN - len );
  }
  else /* si la variable d'env HERMES_CFG_DIR n'existe pas */
  {
    strcpy( cfgpath, cfgfile );
  }

  fd = (int)open( cfgpath, O_RDONLY );

  if ( fd <= 0 )
  {
    p = NULL;
    pClassConfig = NULL;
    ReturnCode = -1;
    printf("Hermes - ErrCfgInit() : Impossible de trouver le fichier \"%s\"\n", cfgpath );
  }
  else
  {
    f = (FILE *)fdopen(fd, "r");

    do
    {
      status = fgets(buf, 512, f);

      if ( status != NULL )
      {
        if ( strncmp(buf, ".CLASS", 6) == 0 )
        {
          pNewInfo = (T_ClassInfo *)malloc( sizeof(T_ClassInfo) );

          if ( pNewInfo != NULL )
          {
            s1 = &buf[6];

            while ((*s1 == ' ') || (*s1 == '\t')) s1++; /* on saute les blancs */

            strncpy(pNewInfo->PkgName,s1,MAXNAMELEN);

            pNewInfo->PkgName[MAXNAMELEN-1] = 0;

            s2 = pNewInfo->PkgName;

            while ((*s2 != 0) && (*s2 != ' ') && (*s2 != '\t'))
	    { /* on passe en majuscules */
              if ((*s2 >= 'a') && (*s2 <= 'z')) *s2 = *s2 - ('a' - 'A');
              s1++;
              s2++;
            }

            *s2 = 0;

            while ((*s1 == ' ') || (*s1 == '\t')) s1++; /* on saute les blancs */

            pNewInfo->ClassCode = atoi(s1);

            while ((*s1 != ' ') && (*s1 != '\t') && (*s1 != '\n') && (*s1 != 0) && (*s1 != '\r')) s1++;

            while ((*s1 == ' ') || (*s1 == '\t')) s1++; /* on saute les blancs */

            if ((*s1 == '\r') || (*s1 == '\n') || (*s1 == 0))
	    { /* il n'y a plus d'arguments */
              memcpy(errdir,cfgdir,MAXFILEPATHLEN);
            }
            else
	    { /* il y a un autre argument */
              strncpy(errdir,s1,MAXFILEPATHLEN);

              errdir[MAXFILEPATHLEN-1] = 0;

              s2 = errdir;

              while ((*s2 != 0) && (*s2 != ' ') && (*s2 != '\t') && (*s2 != '\r') && (*s2 != '\n')) s2++;

              *s2 = 0;

#ifndef __VMS
              len = strlen(errdir);

              if ( len != 0 )
              {
                if ( errdir[len - 1] != '/' )
                {
                  if ( len < ( MAXFILEPATHLEN - 1 ) )
	          {
                    errdir[len] = '/';
                    errdir[len + 1] = 0;
                    len++;
                  }
                  else
	          {
                    ReturnCode = -1;
                    printf("Hermes - ErrCfgInit() : Nom du repertoire du fichier des erreurs trop long !\n");
                    free(pNewInfo);
                    pClassConfig = NULL;
                    fclose(f);
                    return( ReturnCode );
                  }
                }
              }
#endif
            }

            if ( (strlen(errdir) + strlen(pNewInfo->PkgName) + 11) < MAXFILEPATHLEN )
            {
              sprintf(pNewInfo->FilePath,"%s%s_err.hermes",errdir,pNewInfo->PkgName);

              p = pClassConfig;

              while ((p != NULL) && (p->ClassCode != pNewInfo->ClassCode) && (strcmp(p->PkgName,pNewInfo->PkgName) != 0))
	      {
                p = p->Next;
              }

              if ( p == NULL ) /* si on n'a pas trouve */
	      {
                pNewInfo->Next = pClassConfig;

                pClassConfig = pNewInfo;
              }
              else if ( p->ClassCode == pNewInfo->ClassCode )
	      {
                printf("Hermes - ErrCfgInit() : plusieurs fois le meme numero de classe\n");
                pClassConfig = NULL;
                free(pNewInfo);
                ReturnCode = -1;
                return( ReturnCode );
              }
              else /* p->PkgName et pNewInfo->PkgName egales */
	      {
                p->ClassCode = pNewInfo->ClassCode;
                memcpy(p->FilePath,pNewInfo->FilePath,MAXFILEPATHLEN);
                free(pNewInfo);
              }
            }
            else
            {
              free(pNewInfo);
              pClassConfig = NULL;
              ReturnCode = -1;
              printf("Hermes - ErrCfgInit() : Nom du repertoire du fichier des erreurs trop long !\n");
              fclose(f);
              return( ReturnCode );
            }
          }
          else
	  {
            pClassConfig = NULL;
            ReturnCode = -1;
            printf("Hermes - ErrCfgInit() : Memoire insuffisante\n");
            fclose(f);
            return( ReturnCode );
          }
        }
        else if ( strncmp(buf,".APPLICATION",12) == 0 )
        {
          s1 = &buf[12];

          while ((*s1 == ' ') || (*s1 == '\t')) s1++; /* on saute les blancs */

          if ( strncmp(s1,ApplicationName,strlen(ApplicationName)) == 0 )
          {
            while ((*s1 != 0) && (*s1 != ' ') && (*s1 != '\t') && (*s1 != '\r') && (*s1 != '\n')) s1++;
            while ((*s1 == ' ') || (*s1 == '\t')) s1++; /* saute les blancs */

            ApplicationCode = atoi(s1);

            found = 1;
          }
        }
      }
    } while ( status != NULL );

    fclose(f);
  }

  envcfgdir = getenv( USER_CFGDIR_ENV_VAR );

  if ( envcfgdir != NULL )
  {
    strncpy( cfgdir, envcfgdir, MAXFILEPATHLEN );

    cfgdir[MAXFILEPATHLEN-1] = 0; /* au cas ou on aurait tronque ... */

    len = strlen(cfgdir);

#ifndef __VMS
    if ( len != 0 )
    {
      if ( cfgdir[len - 1] != '/' )
      {
        if ( len < ( MAXFILEPATHLEN - 1 ) )
	{
          cfgdir[len] = '/';
          cfgdir[len + 1] = 0;
          len = len + 1;
        }
        else
	{
          ReturnCode = -1;
          printf("Hermes - ErrCfgInit() : Nom du repertoire du fichier de config user trop long !\n");
          p = NULL;
          pClassConfig = NULL;
          return( ReturnCode );
        }
      }
    }
#endif

    strcpy( cfgpath, cfgdir );

    strncat( cfgpath, cfgfile, MAXFILEPATHLEN - len );

    fd = (int)open( cfgpath, O_RDONLY );

    if ( fd <= 0 )
    {
      p = NULL;
      pClassConfig = NULL;
      ReturnCode = -1;
      printf("Hermes - ErrCfgInit() : Impossible de trouver le fichier \"%s\"\n", cfgpath );
    }
    else
    {
      f = (FILE *)fdopen(fd, "r");

      do
      {
        status = fgets(buf, 512, f);

        if ( status != NULL )
        {
          if ( strncmp(buf, ".CLASS", 6) == 0 )
          {
            pNewInfo = (T_ClassInfo *)malloc( sizeof(T_ClassInfo) );

            if ( pNewInfo != NULL )
            {
              s1 = &buf[6];

              while ((*s1 == ' ') || (*s1 == '\t')) s1++; /* on saute les blancs */

              strncpy(pNewInfo->PkgName,s1,MAXNAMELEN);

              pNewInfo->PkgName[MAXNAMELEN-1] = 0;

              s2 = pNewInfo->PkgName;

              while ((*s2 != 0) && (*s2 != ' ') && (*s2 != '\t'))
	      { /* on passe en majuscules */
                if ((*s2 >= 'a') && (*s2 <= 'z')) *s2 = *s2 - ('a' - 'A');
                s1++;
                s2++;
              }

              *s2 = 0;

              while ((*s1 == ' ') || (*s1 == '\t')) s1++; /* on saute les blancs */

              pNewInfo->ClassCode = atoi(s1);

              while ((*s1 != ' ') && (*s1 != '\t') && (*s1 != '\n') && (*s1 != 0) && (*s1 != '\r')) s1++;

              while ((*s1 == ' ') || (*s1 == '\t')) s1++; /* on saute les blancs */

              if ((*s1 == '\r') || (*s1 == '\n') || (*s1 == 0))
	      { /* il n'y a plus d'arguments */
                memcpy(errdir,cfgdir,MAXFILEPATHLEN);
              }
              else
	      { /* il y a un autre argument */
                strncpy(errdir,s1,MAXFILEPATHLEN);

                errdir[MAXFILEPATHLEN-1] = 0;

                s2 = errdir;

                while ((*s2 != 0) && (*s2 != ' ') && (*s2 != '\t') && (*s2 != '\r') && (*s2 != '\n')) s2++;

                *s2 = 0;

#ifndef __VMS
                len = strlen(errdir);

                if ( len != 0 )
                {
                  if ( errdir[len - 1] != '/' )
                  {
                    if ( len < ( MAXFILEPATHLEN - 1 ) )
	            {
                      errdir[len] = '/';
                      errdir[len + 1] = 0;
                      len++;
                    }
                    else
	            {
                      ReturnCode = -1;
                      printf("Hermes - ErrCfgInit() : Nom du repertoire du fichier des erreurs trop long !\n");
                      free(pNewInfo);
                      pClassConfig = NULL;
                      fclose(f);
                      return( ReturnCode );
                    }
                  }
                }
#endif
              }

              if ( (strlen(errdir) + strlen(pNewInfo->PkgName) + 11) < MAXFILEPATHLEN )
              {
                sprintf(pNewInfo->FilePath,"%s%s_err.hermes",errdir,pNewInfo->PkgName);

                p = pClassConfig;

                while ((p != NULL) && (p->ClassCode != pNewInfo->ClassCode) && (strcmp(p->PkgName,pNewInfo->PkgName) != 0))
	        {
                  p = p->Next;
                }

                if ( p == NULL ) /* si on n'a pas trouve */
	        {
                  pNewInfo->Next = pClassConfig;
                  pClassConfig = pNewInfo;
                }
                else if ( p->ClassCode == pNewInfo->ClassCode )
	        {
                  printf("Hermes - ErrCfgInit() : plusieurs fois le meme numero de classe\n");
                  pClassConfig = NULL;
                  free(pNewInfo);
                  ReturnCode = -1;
                  return( ReturnCode );
                }
                else /* p->PkgName et pNewInfo->PkgName egales */
	        {
                  p->ClassCode = pNewInfo->ClassCode;
                  memcpy(p->FilePath,pNewInfo->FilePath,MAXFILEPATHLEN);
                  free(pNewInfo);
                }
              }
              else
              {
                free(pNewInfo);
                pClassConfig = NULL;
                ReturnCode = -1;
                printf("Hermes - ErrCfgInit() : Nom du repertoire du fichier des erreurs trop long !\n");
                fclose(f);
                return( ReturnCode );
              }
            }
            else
	    {
              pClassConfig = NULL;
              ReturnCode = -1;
              printf("Hermes - ErrCfgInit() : Memoire insuffisante\n");
              fclose(f);
              return( ReturnCode );
            }
          }
        }
      } while ( status != NULL );

      fclose(f);
    }
  }

  if (!found)
  {
    ReturnCode = -1;
    printf("Hermes - ErrCfgInit() : Application '%s' inconnue\n",ApplicationName);

    if (SendToLogger)
    {
      sprintf(buf,"Unknown application \"%s\"",ApplicationName);
      msgDefineMessage(0,1,MSG_S_ERROR,NULL,buf);
    }
  }

  return( ReturnCode );
}

T_ClassInfo * ErrCfgGet ( void )
{
  return pClassConfig;
}

void SetHermesApplicationCode ( int AppCode )
{
  ApplicationCode = AppCode;
}

int GetHermesApplicationCode ( void )
{
  return ApplicationCode;
}

void SetHermesApplicationName ( char * AppName )
{
  strncpy(ApplicationName,AppName,MAXNAMELEN - 1);
  ApplicationName[MAXNAMELEN - 1] = 0;
}

char * GetHermesApplicationName ( void )
{
  return ApplicationName;
}
