#
#	Script to check various registers from V. Pucknell 26/9/2005
#	Check extended to other registers G. de France 27/9/2005
#
#


set VXIRegisterServer VXI

proc write-VXIregister       {r v} {global VXIRegisterServer; EG "register :name    '/$VXIRegisterServer/$r' :value '$v'"}
proc write-VXIregisters      {r v} {global VXIRegisterServer; EG "register :pattern '/$VXIRegisterServer/$r' :value '$v'"}

proc read-VXIregister        {r}   {global VXIRegisterServer; EG "register :name    '/$VXIRegisterServer/$r' ?value"}
proc read-VXIregisters       {r}   {global VXIRegisterServer; EG "register :pattern '/$VXIRegisterServer/$r' ?names-and-values"}

proc read-VXIattributes      {r}   {global VXIRegisterServer; EG "register :name    '/$VXIRegisterServer/$r' ?object"}



##############   ESS.LT.ValSample   ##############


insert-log "The ESS card should have a Validation Sample time less than 5us"
insert-log "Now checking ESS LT.ValSample registers"

set values [read-VXIregisters ESS*.LT.ValSample]
foreach item $values {
    set register [lindex $item 0]
    set value [lindex $item 1]           ;#  units are ns
    set vl [first [split $value ns]]
    if {$vl >= 5000} {
        insert-log "register $register has value $value which is out of range"
        write-VXIregister $register 5000ns
        set value [read-VXIregister $register]
        insert-log "register $register now has value $value"
    } else {
        insert-log "register $register has value $value which is good"
    }
}

insert-log "Finished checking ESS LT.ValSample registers"


##############   ESS.CCR.SSControl   ##############


insert-log "The Sliding Scale on the ESS card should be disabled"
insert-log "Now checking ESS CCR.SSEnable registers"

# set values [read-VXIregisters ESS*.CCR.SSEnable]
# 
# G. de France: Sept. 26 Name of register changed
#
set values [read-VXIregisters ESS*.CCR.SSControl]
foreach item $values {
    set register [lindex $item 0]
    set value [lindex $item 1]
    if {$value != 0} {
        insert-log "register $register should have value 0 but discovered to have value $value"
        write-VXIregister $register 0
        set value [read-VXIregister $register]
        insert-log "register $register now has value $value"
    } else {
        insert-log "register $register has value $value which is good"
    }
}

# insert-log "Finished checking ESS CCR.SSEnable registers"
insert-log "Finished checking ESS CCR.SSControl registers"


##############   GOCCE.LastCard   ##############


insert-log "Now checking GOCCE Last Card registers"

set values [read-VXIregisters GOCCE*.LastCard]
foreach item $values {
    set register [lindex $item 0]
    set value [lindex $item 1]
    set attributes [read-VXIattributes $register]
    if {[string match *-i1* $attributes] == 1} then {     ;# look for an attribute -i1
        if {$value != 1} then {
            insert-log "$register should have value 1 but discovered to have value $value"
            write-VXIregister $register 1
            set value [read-VXIregister $register]
            insert-log "register $register now has value $value"
        } else {
            insert-log "$register has value 1 which is good"
        }
    } else {                           ;#  if no attribute -i1 then register value should be 0
        if {$value != 0} then {
            insert-log "$register should have value 0 but discovered to have value $value"
            write-VXIregister $register 0
            set value [read-VXIregister $register]
            insert-log "register $register now has value $value"
        } else {
            insert-log "register $register has value 0 which is good"
        }
   }
}

insert-log "Finished checking GOCCE Last Card registers"

##############   GOCCE.DSP.Reset   ##############

insert-log "Now checking GOCCE.DSP.Reset registers"

set values [read-VXIregisters GOCCE*.DSP.Reset]
foreach item $values {
    set register [lindex $item 0]
    set value [lindex $item 1]
    set attributes [read-VXIattributes $register]
    if {[string match *-i1* $attributes] == 1} then {     ;# look for an attribute -i1
        if {$value != 1} then {
            insert-log "$register should have value 1 but discovered to have value $value"
            write-VXIregister $register 1
            set value [read-VXIregister $register]
            insert-log "register $register now has value $value"
        } else {
            insert-log "$register has value 1 which is good"
        }
    } else {                           ;#  if no attribute -i1 then register value should be 0
        if {$value != 0} then {
            insert-log "$register should have value 0 but discovered to have value $value"
            write-VXIregister $register 0
            set value [read-VXIregister $register]
            insert-log "register $register now has value $value"
        } else {
            insert-log "register $register has value 0 which is good"
        }
   }
}

insert-log "Finished checking GOCCE.DSP.Reset registers"


##############   GOCCE.ResetGIR   ##############

insert-log "Now checking GOCCE.ResetGIR registers"

set values [read-VXIregisters GOCCE*.ResetGIR]
foreach item $values {
    set register [lindex $item 0]
    set value [lindex $item 1]
    set attributes [read-VXIattributes $register]
    if {[string match *-i1* $attributes] == 1} then {     ;# look for an attribute -i1
        if {$value != 1} then {
            insert-log "$register should have value 1 but discovered to have value $value"
            write-VXIregister $register 1
            set value [read-VXIregister $register]
            insert-log "register $register now has value $value"
        } else {
            insert-log "$register has value 1 which is good"
        }
    } else {                           ;#  if no attribute -i1 then register value should be 0
        if {$value != 0} then {
            insert-log "$register should have value 0 but discovered to have value $value"
            write-VXIregister $register 0
            set value [read-VXIregister $register]
            insert-log "register $register now has value $value"
        } else {
            insert-log "register $register has value 0 which is good"
        }
   }
}

insert-log "Finished checking GOCCE.ResetGIR registers"  


##############   GOCCE.CRC.SSEnable   ##############


insert-log "The Sliding Scale on the GOCCE card should be disabled"
insert-log "Now checking GOCCE CRC.SSEnable registers"

set values [read-VXIregisters GOCCE*.CRC.SSEnable]
foreach item $values {
    set register [lindex $item 0]
    set value [lindex $item 1]
    if {$value != 0} {
        insert-log "register $register should have value 0 but discovered to have value $value"
        write-VXIregister $register 0
        set value [read-VXIregister $register]
        insert-log "register $register now has value $value"
    } else {
        insert-log "register $register has value $value which is good"
    }
}

insert-log "Finished checking GOCCE CRC.SSEnable registers"
