#    d2vbdiag.tcl  -  D2VB module diagnostic frame


set frame d2vbdiag
set frametitle "D2VB Diagnostics Frame"
set icontitle "D2VBdiag"

set mname "D2VB"

if {[info exists env(D2VBserver)]} then {set resource $env(D2VBserver)} \
    else {set resource VME}

#    load the common f2vb/d2vb source

loadsource x2vbdiag

set DataWidth 32


set F2VB_registerbase 0x00e00000
set F2VB_listrambase  0x00e00800


proc get-config {} {
    global env configarray

   set Configdir [file join $env(MIDASAPPLBASE) config]
   set experiment [file tail $env(EXPERIMENTBASE)]
   set fname [file join $Configdir [set experiment]_Current_VXI_Configuration]

   set z [catch {open $fname r} f]
   if {$z != 0} then {midas-warning "Unable to read file $fname\n$f"; return}
   set config [split [read $f] \n]
   foreach line $config {
       if {[string range $line 0 0] != "#"}  {
           set configarray([lindex $line 3])  [lindex $line 0]
       }
   }
   close $f
}

proc define-x2vb-registers {} {
   global F2VB_registerbase F2VB_listrambase
   global resource mname dumpwords

   set register /$resource/$mname.D*
   set-footer "deleting register $register"
   EG "register :pattern '$register' !delete"

   define-register xVME DCSR  "-a $F2VB_registerbase  -o 0   -w 24 -d 16 -z x -p rw"
   define-register xVME DCCR  "-a $F2VB_registerbase  -o 2   -w 24 -d 16 -z x -p rw"
   define-register xVME DICR  "-a $F2VB_registerbase  -o 6   -w 24 -d 16 -z x -p rw"
   define-register xVME DIVR  "-a $F2VB_registerbase  -o 8   -w 24 -d 16 -z x -p rw"
   define-register xVME DLRAR "-a $F2VB_registerbase  -o 10  -w 24 -d 16 -z x -p ro"
   define-register xVME DDRAR "-a $F2VB_registerbase  -o 12  -w 24 -d 16 -z x -p rw"
   define-register xVME DCBRA "-a $F2VB_registerbase  -o 16  -w 24 -d 32 -z x -p ro"
   define-register xVME DCBWA "-a $F2VB_registerbase  -o 20  -w 24 -d 32 -z x -p ro"

   define-register bVME Dbram "-a [expr [read-register DDRAR] << 16] -w 32 -d 32 -z x -p rw -l $dumpwords"
   define-register bVME Dlram "-a $F2VB_listrambase   -w 24 -d 16 -z x -p rw -l 96"


    get-config
}

proc define-register {class name attributes} {
   global resource mname

   set register /$resource/$mname.$name
   set-footer "defining register $register"
   EG "register :name '$register' :class '$class' !new"
   EG "register :name '$register' :object '$attributes'"
   clear-footer
}


proc add-format-option {} {
     :choice-stack FORMATOPT :label "format option" :strings [list 0 1 2] :value 2
}


proc format_the_dump {} {
  global dumpdata configarray
  global dumpopt dumpwords
  global x2VBmemOffset

#    format dumpdata held in global variable dumpdata

   set-footer "formatting dump"

   set frame "dump"
   :frame [self frame-name].$frame :label  "D2VB Dump"  :transient T

   switch $dumpopt {
   0  {
        :galley G :rows 50 :cols 50 :deaf T :strings "" :font fixed
        :show T
        :modify G :strings "raw dump starting at word offset 0\n"
        :modify G :strings $dumpdata
      }

   1  {
        :galley G :rows 50 :cols 50 :deaf T :strings "" :font fixed
        :show T
        :modify G :strings "dump starting at word offset $x2VBmemOffset for $dumpwords words\n"
        set dump ""
        for {set i 0} {$i < $dumpwords} {incr i} {
           set word [lindex $dumpdata $i]
           set group [expr $word >> 16 & 255]
           set item  [expr $word >> 24 & 63]
           if {$group >= 248 && $item == 63} then {
#    last end token for event
             append dump "$word "
             :modify G :strings $dump
             :modify G :strings "\n\n"
             set dump ""
           } else {
             append dump "$word "
           }
        }
        :modify G :strings $dump
     }
   2  {
        :galley G :rows 50 :cols 60 :deaf T :strings "" :font fixed
        :show T
        :modify G :strings "dump starting at word offset $x2VBmemOffset for $dumpwords words\n"
        for {set i 0} {$i < $dumpwords} {incr i} {
           set word [lindex $dumpdata $i]
           set label [expr $word >> 16 & 0x00003fff]
           set group [expr $word >> 16 & 255]
           set item  [expr $word >> 24 & 63]
           set qbits [expr $word >> 30 & 3]
           set data  [expr $word & 0x0000ffff]
           set title "unknown item label"
           if {$group < 248} then { 
               if {[info exists configarray($label)]} then \
                   {set title $configarray($label)}
           } else {
#               tokens 
              if {$item <= 15} then {set title "start token"} \
              elseif {$item <= 31} then {set title "event number lsw"} \
              elseif {$item <= 47} then {set title "event number msw"} \
              else {set title "end token"}
           }
           :modify G :strings "[format "%10s %3d %3d %4d %6x %24s" $word $qbits $item $group $data $title] \n"
           if {$item == 63} then {:modify G :strings "\n"}
       }
     }

   }
   :modify G :see 0
   clear-footer
}
