# goccescalers.tcl---GOCCE card Scaler module control frame


set frame GOCCESCALERS
set frametitle "GOCCE Scaler Monitor"
set icontitle "GOCCESCALERS"

set NUM 16

set modulelist [inform gocce inquire chanlist]
set maxmodule [llength $modulelist]
set maxcounter [expr $maxmodule * $NUM]

set module [lindex $modulelist 0]

set rates 0
set rateperiod 10
set elapseperiod [expr 1000 * $rateperiod]

for {set c 0} {$c < $maxcounter} {incr c} {set data($c) 0; set data1($c) 0; set data2($c) 0; set rate($c) 0}
set rateid 0

proc read-scaler {i}  {
    global modulelist
 
    set seg [expr $i & 3]
    set xtal [lindex [list A B C D] [expr ($i >> 2) & 3]]
    set chan [expr $i >> 4]

    set module GOCCE[lindex $modulelist $chan]
    set server [send vxicontrol VXImodule2crate $module]

    set register /$server/$module.Xtal$xtal.Seg$seg.Scaler

    set i [EG "register :name '$register' ?value"]

    return $i
}

proc click_rates {v} {
   global rates rateperiod rateid data data1 data2 maxcounter
   set rates $v
   :modify RATES :value $rates
   if {$rates == 0} then {
       after cancel $rateid
       set rateid 0
       display_rates
   } else {
      if {$rateid != 0} then {return}
      display_counters
      for {set c 0} {$c < $maxcounter} {incr c} {set data2($c) $data($c)}
      set rateid [after [expr $rateperiod * 1000] rates_update]
   }
}

proc rates_update {} {
   global rates rateperiod rateid data data1 data2 maxcounter
   if {$rates == 0} then {return}

   if {[:query_Mapped .]} then {display_counters}
   for {set c 0} {$c < $maxcounter} {incr c} {set data1($c) $data($c)}
   if {[:query_Mapped .]} then {display_rates}
   for {set c 0} {$c < $maxcounter} {incr c} {set data2($c) $data1($c)}
   set rateid [after [expr $rateperiod * 1000] rates_update]
}

proc display_counters {} {
   global NUM modulename module data maxcounter EGreport

   set EGreport -1
   set-busy
   
   for {set c 0} {$c < $maxcounter} {incr c} {
       set module [expr ($c / $NUM) + 1]
       set i [expr $c - (($module - 1) * $NUM)]
#    read it
       set data($c) [expr [read-scaler $c] & 0x00ffffff]
       :modify C[set c]DATA  :value $data($c)
   }
   clear-busy
}

proc display_rates {} {
   global rate data1 data2 rates elapseperiod maxcounter

   clear-footer
   for {set c 0} {$c < $maxcounter} {incr c} {
       if {$rates == 0} then {
          set data 0
       } else {
          set data [expr (($data1($c) - $data2($c)) * 1000) / $elapseperiod]
       }
       set rate($c) $data
       :modify C[set c]RATE  :value $rate($c)
   }
   set-footer "Rates last updated [date]"
}

proc click-redisplay {} {display_counters}  

proc click arg {
   global debug actionlist
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]
   if {$c==":quit"} then {click-quit; return}
   switch $w {

      RATES  {click_rates $v}
      REDISPLAY {click-redisplay}

      default           {
           set p "click_[lowercase $w]"
           if {[info procs $p] == "$p"} then {$p} else {insert-debug "$arg"}
      }
   }
}

proc make-frame {} {
  global env maxcounter rates modulelist

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/vme_module" \
      :icon-name "[self icon-title]" :resize T 
        
  
   :panel MODULE :layout T :padx 10 :pady 10
      :layout H; :row-gap 10

      :message M1 :label "Counter Values"

      for {set i 0} {$i < $maxcounter} {incr i} {
        if {[expr $i/8*8] == $i} then {:next-row T}
        set label [lindex $modulelist [expr $i >> 4]].[expr $i & 15]
        :text C[set i]DATA :label $label: :valuex 5c :w 10 :value 0 :deaf T
      }

#     :next-row T
#     :message M2 :label "Rate Values"
#
#      for {set i 0} {$i < $maxcounter} {incr i} {
#        if {[expr $i/8*8] == $i} then {:next-row T}
#        :text C[set i]RATE :label $i: :valuex 3c :w 10 :value 0 :deaf T
#      }

   :panel CLICK :layout T :padx 10
      :layout H; :col-gap 10

      :button REDISPLAY  :label "Update Counters" 
#      :checkbox RATES    :label "Enable Rates" :value $rates

   :show T
}
