#    hist task code overlay for detector type "space"


set lowchannel  0
set highchannel 15
set maxchannel  15

set spacenames   {T30 T60 T90 Tmax Pos Eu Em El n1 n2}

set space 0

set xtalnames {A B C D}
set xtal 15
set segnames {0 1 2 3}
set seg 15

proc make-overlaypanel {} {
   global spacenames girnames
   global segnames seg xtalnames xtal qnames

      :layout V; :row-gap 10; :col-gap 20

      :message DET-M1   :label "SPACE overlay applied"

      :number DET-LOW  :label "Low channel:"  :w 5 :valuex 15c
      :number DET-HIGH :label "High channel:" :w 5 :valuex 15c

      :choice DET-XTAL  :label "Crystal:"       :valuex 15c :exclusive F :strings $xtalnames :value $xtal
      :choice DET-SEG   :label "Segment:"       :valuex 15c :exclusive F :strings $segnames :value $seg

      :next-row T
      :layout H
      :choice DET-SPACE   :label "Items" :valuex 8c  :exclusive F :strings $spacenames  :value 0
}



#    this proc handles events from widgets created in the proc make-overlaypanel
#    The widgets DET-LOW; DET-HIGH and DET-QUAL are handled in the main click procedure
#    and are not required here

proc click-overlay {w v} {

   switch -glob $w {
      DET-SPACE         {click-adc $v space}
      DET-SEG           {click-segment $v}
      DET-XTAL          {click-crystal $v}
      default      {puts stdout $arg}
   }
}

proc repaint_overlay {} {
   global spectrumlist
   global space gir seg xtal qualifier

   if {[llength $spectrumlist] > 0} then {
       :modify DET-SPACE    :value 0
       :modify DET-SEG      :value 0
       :modify DET-XTAL     :value 0
   } else {
       :modify DET-SPACE    :value $space
       :modify DET-SEG      :value $seg
       :modify DET-XTAL     :value $xtal
   }
}


proc get-spectrum-list {} {
        global spectrumlist
        global spacenames space 
        global lowchannel highchannel
        global seg segnames
        global xtal xtalnames

        if {$spectrumlist != {}} then {return $spectrumlist};  # return names selected from spectrum list

        set seglist [list-take $segnames  [bits-in $seg]]
        set xtallist [list-take $xtalnames [bits-in $xtal]]

        set list {}

        for {set i $lowchannel} {$i<=$highchannel} {incr i} {
# append  spectrum names

            foreach x $xtallist {
            foreach s $seglist {
            foreach adc [list-take $spacenames  [bits-in $space]]  {
                 lappend list SPACE$i.Xtal$x.Seg$s.$adc
            } } }
        }
        return $list
}

proc click-segment {v} {
   global spectrumlist
   global seg
   set seg $v
   if {$spectrumlist!={}} then {paint-histlist; repaint}
}

proc click-crystal {v} {
   global spectrumlist
   global xtal
   set xtal $v
   if {$spectrumlist!={}} then {paint-histlist; repaint}
}

proc range-of n {

    set fields [split $n "."]
    if {[string match "SPACE*" [first $fields]]} then {
        switch -glob [fourth $fields]  {
            Pos    -
            Tmax   -
            T30    -
            T60    -
            T90       {return 16384}
            default   {return 65536}
        }
     }
}
