#
#    Task to obtain various rate information and to monitor the system

#    1/10/97

#    obtains trigger scaler rates
#    monitors state of tape drives and can provide warning and/or halt DA
#    when approaching end of tape

set debug 0
set started 0

set frame RATES
set frametitle "Rates and Monitoring"
set icontitle Rates

set trigrateperiod 5
set singrateperiod 5
set taperateperiod 15
set tapemonitorperiod 15

set trigratesstarted 0
set singratesstarted 0
set taperatesstarted 0
set tapemonitorstarted 0

set autorestartenable 0
set autorestartcount 0

#    Trigger rates
#    for Trigger MK1 there are 7 scalers (FTR1 => FTR4, FT, VAL, Test)
#    for Trigger MK2 there are 8 scalers (Ftr1 => Ftr4, Ft, Val, Rdout, User)

set Trigpres 0
set triggerrates {0 0 0 0 0 0 0 0}
set TrigMK1Scalers {FTR1 FTR2 FTR3 FTR4 FT Val Test Test}
set TrigMK1ScalerNames {FTR1 FTR2 FTR3 FTR4 FT Val Test Test}
set TrigMK2Scalers {Scaler1 Scaler2 Scaler3 Scaler4 Scaler5 Scaler6 Scaler7 Scaler8}
set TrigMK2ScalerNames {Ftr1 Ftr2 Ftr3 Ftr4 Ft Val Rdout User}
set ScalersList $TrigMK1Scalers
set ScalerNames $TrigMK1ScalerNames

set nsteps 2
foreach scaler $TrigMK1Scalers {set [set scaler]count 0; set [set scaler]counts [list 0 0 0 0 0 0 0 0 0 0]}
foreach scaler $TrigMK2Scalers {set [set scaler]count 0; set [set scaler]counts [list 0 0 0 0 0 0 0 0 0 0]}

#    Singles
#    assume up to 2 systems

set singlesrates {0 0}

#    Tape drive rates
#    just handle the first 4 drives

set tapestate 0
set taperates {0 0 0 0}
set tapeusedpercent {0 0 0 0}

if {$env(MASTERCONSOLE) == "ALL"  || $env(MASTERCONSOLE) == [exec hostname]} then {
    set tapewarn 90
    set tapehalt 95
} else {
    set tapewarn disabled
    set tapehalt disabled
}

set CurrentVXIServer  VXI
set TrigServer vxi1
set TrigModule Trig
set Trigpres 0

set EGreport -1
set EGallow 0x0

proc read-vxi-reg {register} {
   global CurrentVXIServer TrigServer TrigModule
   if {[first [split $register .]] == $TrigModule} then {
      set VXIregister /$TrigServer/$register
   } else {
      set VXIregister /$CurrentVXIServer/$register
   }
   EG "register :name '$VXIregister' ?value"
}

proc write-vxi-reg {register value} {
   global CurrentVXIServer TrigServer TrigModule
   if {[first [split $register .]] == $TrigModule} then {
      set VXIregister /$TrigServer/$register
   } else {
      set VXIregister /$CurrentVXIServer/$register
   }
   EG "register :name '$VXIregister' :value '$value'"
}

proc make-frame {} {
  global env
  global triggerrates singlesrates taperates tapeusedpercent
  global autorestartenable FtrThresh TrigThresh

  :frame "[self frame-name]" :label "[self frame-title]" :show-footer T  :resize F \
      :icon-image "@$env(ICONHOME)/default" \
      :icon-name "[self icon-title]"

    :panel PANEL1 :layout T :padx 10 :pady 10
      :layout H; :row-gap 5; :col-gap 10
      :message M1    :label "Rate Monitoring" :bold T :valuex 25c
      :next-row T
      :message TRIGRATES :label "Trigger scaler rates (counts/sec) -  $triggerrates"
      :next-row T
      :message SINGRATES :label "Singles buffer rates (buffers/sec) -  $singlesrates"
      :next-row T
      :message TAPERATES :label "Tape data rates (Kbytes/sec) -  $taperates"
      :next-row T
      :message M2    :label "Tape Drive Monitoring" :bold T :valuex 25c
      :next-row T
      :message TAPEUSED :label "Tape used (%) -  $tapeusedpercent"
      :next-row T
      :choice-stack TAPEWARN   :label "Warning when tape used is (%)" \
           :strings [list 80 85 90 92 94 96 98 disabled] :value 7
      :choice-stack TAPEHALT   :label "Halt when tape used is (%)" \
           :strings [list 90 92 94 95 96 98 99 disabled] :value 7

    :panel BUTTONS :layout B :padx 10 :pady 10 :after FOOTER
    :layout H; :col-gap 10; :row-gap 10
      :button STARTTRIGR   :label "Start trigger rate monitoring"
      :button STARTSING   :label "Start Singles rate monitoring"
      :next-row T
      :button STARTTAPER   :label "Start tape rate monitoring"
      :button STARTM       :label "Start tape monitoring"
      :next-row T
      :message MA1 :label "Auto Restart" :bold T
      :row-gap 0
      :next-row T
      :message MA2 :label "Performs DA STOP then DA GO if sum of Trigger scalers 1=>4 > $FtrThresh and \
average of Trigger scalers 5=>8 is < $TrigThresh"
      :next-row T
      :checkbox AUTORESTART :label "Auto restart enable" :value $autorestartenable
      :row-gap 10
      :next-row T
      :button DSTATSON :label "Start ExoGAM Data Quality monitor"
      
   :show T
}


proc click arg {
  global tapewarn tapehalt trigratesstarted singratesstarted taperatesstarted tapemonitorstarted
  global autorestartenable
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]
   if {$c==":quit"} then {return [click-quit]}
   switch $w  {
      TAPEWARN  {set tapewarn $i}
      TAPEHALT  {set tapehalt $i}
      STARTTRIGR    {
         if {$trigratesstarted == 0} then {start-trig-rates} else {stop-trig-rates}
      }
      STARTSING    {
         if {$singratesstarted == 0} then {start-sing-rates} else {stop-sing-rates}
      }
      STARTTAPER    {
         if {$taperatesstarted == 0} then {start-tape-rates} else {stop-tape-rates}
      }
      STARTM    {
         if {$tapemonitorstarted == 0} then {start-tapemonitor} else {stop-tapemonitor}
      }
      AUTORESTART {set autorestartenable $v}
      DSTATSON {inform stats start}
      default {click-[string tolower $w] $v}
   }
}

proc redisplay {} {
  global triggerrates trigratesstarted
  global singlesrates singratesstarted
  global taperates taperatesstarted tapeusedpercent tapemonitorstarted

  if {![:query_Mapped .]} {return}

  :modify TRIGRATES :label "Trigger scaler rates (counts/sec) -  $triggerrates"
  :modify SINGRATES :label "Singles buffer rates (counts/sec) -  $singlesrates"
  :modify TAPERATES :label "Tape data rates (Kbytes/sec) -  $taperates"
  :modify TAPEUSED  :label "Tape used (%) -  $tapeusedpercent"
  if {$trigratesstarted == 0} then {
     :modify STARTTRIGR  :label "Start trigger rate monitoring"
  } else {
     :modify STARTTRIGR  :label "Stop trigger rate monitoring"
  }
  if {$singratesstarted == 0} then {
     :modify STARTSING  :label "Start singles rate monitoring"
  } else {
     :modify STARTSING  :label "Stop singles rate monitoring"
  }
  if {$taperatesstarted == 0} then {
     :modify STARTTAPER  :label "Start tape rate monitoring"
  } else {
     :modify STARTTAPER  :label "Stop tape rate monitoring"
  }
  if {$tapemonitorstarted == 0} then {
     :modify STARTM  :label "Start tape monitoring"
  } else {
     :modify STARTM  :label "Stop tape monitoring"
  }
  set-footer "Updated [date]"
}

#    check for hung DA system
#    if trigger scaler 1 > 1000  (ie data is available)  but scalers 5,6,7 and 8
#    are all < 1 (ie 0) then restart DA system

set FtrThresh 1000
set TrigThresh 1

proc autorestartmonitor {} {
   global autorestartenable autorestartcount triggerrates
   global FtrThresh TrigThresh
   
   if {$autorestartenable == 0} then {return}

   set ftr1_rate [lindex $triggerrates 0]
   set ftr2_rate [lindex $triggerrates 1]
   set ftr3_rate [lindex $triggerrates 2]
   set ftr4_rate [lindex $triggerrates 3]
   set ftr_rate [expr $ftr1_rate + $ftr2_rate + $ftr3_rate + $ftr4_rate]     

   set scaler5 [lindex $triggerrates 4]
   set scaler6 [lindex $triggerrates 5]
   set scaler7 [lindex $triggerrates 6]
   set scaler8 [lindex $triggerrates 7]
   set trig_rate [expr [expr $scaler5 + $scaler6 + $scaler7 + $scaler8] / 4]

   if {$ftr_rate > $FtrThresh && $trig_rate < $TrigThresh} then {
       incr autorestartcount
       if {$autorestartcount >= 3 } then {inform sys do_stopandgo; set autorestartcount 0}
   } else {
       set autorestartcount 0
   }
}

proc update-trigger-rates {} {
  global Trigpres TrigServer TrigModule triggerrates trigrateperiod nsteps
  global ScalersList TrigMK1Scalers TrigMK2Scalers ScalerNames TrigMK1ScalerNames TrigMK2ScalerNames

    set Trigpres [send vxicontrol inquire Trigpres]
    if {$Trigpres == 0} then {
       zero-rates
       return
    }

    set TrigServer [send vxicontrol inquire TrigServer]
    set TrigModule [send vxicontrol inquire TrigModule]
    if {$TrigModule == "Trig"} then {
       set ScalersList $TrigMK1Scalers
       set ScalerNames $TrigMK1ScalerNames
    } else {
       set ScalersList $TrigMK2Scalers
       set ScalerNames $TrigMK2ScalerNames
    }

    set z [catch {set state [read-vxi-reg $TrigModule.VXI.Go]} m]
    if {$z != 0} then {insert-log "Reading Trigger VXI.Go register failed code $m"}

    if {$z != 0 || $state == 0} then {
#    STOPPed - so all rates are zero
       zero-rates
    } else {
#    Going
       set triggerrates {}
       if {$TrigModule == "Trig"} then {
          write-vxi-reg $TrigModule.Scalers.Xfer 1
          write-vxi-reg $TrigModule.Scalers.Xfer 0
       } else {

          set z [catch {set state [write-vxi-reg $TrigModule.XferScalers 0]} m]
          if {$z != 0} then {insert-log "Writing 0 to Trigger XferScalers register failed code $m"}

          set z [catch {set state [write-vxi-reg $TrigModule.XferScalers 1]} m]
          if {$z != 0} then {insert-log "Writing 1 to Trigger XferScalers register failed code $m"}

       }
       foreach scaler $ScalersList {
          upvar #0 [set scaler]count count
          upvar #0 [set scaler]counts counts
          set s 0
          if {$TrigModule == "Trig"} then {
             set s [read-vxi-reg $TrigModule.$scaler.Scaler]
          } else {
             set z [catch {set s [read-vxi-reg $TrigModule.$scaler]} m]
             if {$z != 0} then {
                 insert-log "Reading Trigger Scaler register failed code $m"
                 set s 0
             }
          }
          if {[first $s] == "can't"} then {set s 0}
          if {[last $counts] == 0} then {
             set counts {}
             for {set i 0} {$i < $nsteps} {incr i} {lappend counts $s}
          } else {
             set counts [lreplace $counts 0 0]
             lappend counts $s
          }
          for {set i 1} {$i < $nsteps} {incr i} {
             set diff$i [expr [lindex $counts $i] -  [lindex $counts [expr $i - 1]]]
          }
          set diffsum 0
          set steps 0
          for {set i 1} {$i < $nsteps} {incr i} {
             if {[set diff$i] > 0} then {
                 set diffsum [expr $diffsum + [set diff$i]]
                 incr steps
             }    
          }
          if {$steps > 0} then {
              set rate [expr [expr $diffsum / $steps] / $trigrateperiod]
          } else {
              set rate 0
          }
          lappend triggerrates "$rate"
          set count $s
       }
    }
#  insert-debug "$triggerrates"
}

proc get-singles-processors {} {inform singles inquire ProcessorList}

proc update-singles-rates {} {
   global singlesrates singrateperiod nsteps

   set processors [get-singles-processors]

   if {$processors == ""} then {
      zero-sing-rates
      return
    }

       set singlesrates {}
       foreach proc $processors {
          upvar #0 [set proc]count count
          upvar #0 [set proc]counts counts
# A buffer contains 128 counts
          set c [inform singles get-buffers-sent $proc]
          if {[scan-int $c] == "???"} then {set c 0}
          set s [expr $c * 128]
          if {[last $counts] == 0} then {
             set counts {}
             for {set i 0} {$i < $nsteps} {incr i} {lappend counts $s}
          } else {
             set counts [lreplace $counts 0 0]
             lappend counts $s
          }
          for {set i 1} {$i < $nsteps} {incr i} {
             set diff$i [expr [lindex $counts $i] -  [lindex $counts [expr $i - 1]]]
          }
          set diffsum 0
          set steps 0
          for {set i 1} {$i < $nsteps} {incr i} {
             if {[set diff$i] > 0} then {
                 set diffsum [expr $diffsum + [set diff$i]]
                 incr steps
             }    
          }
          if {$steps > 0} then {
              set rate [expr [expr $diffsum / $steps] / $singrateperiod]
          } else {
              set rate 0
          }
          lappend singlesrates "$rate"
          set count $s
       }
}

proc update-tape-rates {} {
  global tapestate taperates

   set tapestate [inform tape read-egstate]
   if {$tapestate == 0} then {
#    STOPped - set rates to zero
       set taperates {0 0 0 0}
   } else {
#    Going
       inform tape update-information
       set taperates [inform tape inquire taperates]
   }
#  insert-debug "$taperates"
}

proc tape-warning {} {
  global tapewarn tapehalt tapestate tapeusedpercent
   set tapeusedpercent {}
   set tapestate [inform tape read-egstate]
   if {$tapestate == 0} then {return}
   set tapeusedpercent [inform tape inquire tapepercent]
   for {set i 0} {$i < [llength $tapeusedpercent]} {incr i} {
      set percent [lindex $tapeusedpercent $i]
      if {$tapewarn != "disabled" && $percent >= $tapewarn} then {
#    generate warning
         warning-msg "You are near the end of tape - $percent% used"
         bell
      }
      if {$tapehalt != "disabled" && $percent >= $tapehalt} then {
#    halt DA
         warning-msg "You are too near the end of tape ($percent% used)\n****Data Acquisition STOPPED****"
         bell;bell;bell;bell;bell
         inform sys "click-exogam 0"
         insert-log "****Data Acquisition auto-STOPPED near end of tape****"
         break
      }
   }
}

proc warning-msg {msg} {

  :frame [self frame-name].warning :label "Tape Warning" :show-footer T :resize F :transient T

    :panel A

      :font local message b :pxlsz 24
      :message M :label "Warning --- $msg" :bold T

      :show T
}

proc zero-rates {} {
   global triggerrates ScalersList
   foreach scaler $ScalersList {
      upvar #0 [set scaler]count count
      upvar #0 [set scaler]counts counts
      set count 0
      set counts [list 0 0 0 0 0 0 0 0 0 0]
   }
   set triggerrates {0 0 0 0 0 0 0 0}
}

proc start-trig-rates {} {
  global trigrateperiod trigratesstarted
   if {$trigratesstarted == 1} then {return}
   zero-rates
   zero-sing-rates
   set taperates {0 0 0 0}
   update-trigger-rates
   after [expr $trigrateperiod * 1000] continue-trig-rates
   set trigratesstarted 1
   redisplay
}

proc stop-trig-rates {} {
   global trigratesstarted
   set trigratesstarted 0
   zero-rates
   redisplay
}

proc zero-sing-rates {} {
   global singlesrates
   set processors [get-singles-processors]
   foreach proc $processors {
      upvar #0 [set proc]count count
      upvar #0 [set proc]counts counts
      set count 0
      set counts [list 0 0 0 0 0 0 0 0 0 0]
   }
   set singlesrates {0 0}
}

proc start-sing-rates {} {
  global singrateperiod singratesstarted
   if {$singratesstarted == 1} then {return}
   zero-sing-rates
   update-singles-rates
   after [expr $singrateperiod * 1000] continue-sing-rates
   set singratesstarted 1
   redisplay
}

proc stop-sing-rates {} {
   global singratesstarted
   set singratesstarted 0
   zero-sing-rates
   redisplay
}

proc start-tape-rates {} {
  global taperateperiod taperatesstarted taperates
   if {$taperatesstarted == 1} then {return}
   set taperates {0 0 0 0}
   update-tape-rates
   after [expr $taperateperiod * 1000] continue-tape-rates
   set taperatesstarted 1
   redisplay
}

proc stop-tape-rates {} {
   global taperatesstarted taperates
   set taperatesstarted 0
   set taperates {0 0 0 0}
   redisplay
}

proc start-tapemonitor {} {
  global tapemonitorperiod tapemonitorstarted
   if {$tapemonitorstarted == 1} then {return}
   after [expr $tapemonitorperiod * 1000] continue-tape-monitor
   set tapemonitorstarted 1
   tape-warning
   redisplay
}

proc stop-tapemonitor {} {
   global tapemonitorstarted tapeusedpercent
   set tapemonitorstarted 0
   set tapeusedpercent {}
   redisplay
}

proc continue-trig-rates {} {
  global trigrateperiod trigratesstarted
   if {$trigratesstarted == 0} then {return}
   after [expr $trigrateperiod * 1000] continue-trig-rates
   update-trigger-rates
   autorestartmonitor
   redisplay
}

proc continue-sing-rates {} {
  global singrateperiod singratesstarted
   if {$singratesstarted == 0} then {return}
   after [expr $singrateperiod * 1000] continue-sing-rates
   update-singles-rates
   redisplay
}

proc continue-tape-rates {} {
  global taperateperiod taperatesstarted
   if {$taperatesstarted == 0} then {return}
   after [expr $taperateperiod * 1000] continue-tape-rates
   update-tape-rates
   redisplay
}

proc continue-tape-monitor {} {
   global tapemonitorperiod tapemonitorstarted
   if {$tapemonitorstarted == 0} then {return}
   after [expr $tapemonitorperiod * 1000] continue-tape-monitor
   tape-warning
   redisplay
}
