
#   sys.tcl     ExoGam Version     May 04

set frametitle "ExoGam Data Acquisition System Control"

set VERSION "EXOGAM"

set env(RDOUThdwre) D2VB

set CAMACPresent 0
set VMEPresent 1
set VXIPresent 1

set TSPresent 1

set CAMACEnable 0
set VMEEnable 1
set VXIEnable 1

set RDOUTEnable 1

set histEnable 1
set HistMode 1

set txEnable 1

set MaxTries 100

set autoperiod 5000
set autocount 10

set CheckDAPeriod 10

if {$env(MASTERCONSOLE) == "ALL"  || $env(MASTERCONSOLE) == [exec hostname]} then {
   set VXIEnable 1
   set VMEEnable 1
   set RDOUTEnable 1
   set TSEnable 0   ;#     not controlled by default
} else {
   set VXIEnable 0
   set VMEEnable 0
   set RDOUTEnable 0
   set TSEnable 0
}


set DAProgram VXIExecV16
loadsource  sys.common

lappend setuplist [list "Make this display a system console" do_system_console]
lappend setuplist [list "Make this display a slave console"  do_slave_console]
lappend setuplist [list "Make Configuration File"            Make_Configuration_File]
lappend setuplist [list "Force Spectrum Map needed"          map_needed]

lappend setuplist [list "Show histogram name translations" "Show_Xlate"]
lappend setuplist [list "Edit histogram translation file"  "edit_xlate"]
lappend setuplist [list "2D Histogram Configuration"       "2D_Histogram_Configuration"]
lappend setuplist [list "Create 2D Histograms"             "Create_2D_Histograms"]

loadsource sys.2D    ;# for 2D histogrammming feature 

lappend actionlist [list "Start Trigger rate monitoring"     "send rates start-trig-rates"]
lappend actionlist [list "Stop Trigger rate monitoring"      "send rates stop-trig-rates"]
lappend actionlist [list "Start Singles rate monitoring"     "send rates start-sing-rates"]
lappend actionlist [list "Stop Singles rate monitoring"      "send rates stop-sing-rates"]
lappend actionlist [list "Start Tape rate monitoring"        "send rates start-tape-rates"]
lappend actionlist [list "Stop Tape rate monitoring"         "send rates stop-tape-rates"]
lappend actionlist [list "Start Tape monitoring"             "send rates start-tapemonitor"]
lappend actionlist [list "Stop Tape monitoring"              "send rates stop-tapemonitor"]

set Supported_Modules {}   ;#  since not DA from VME modules

set SetupMessage "\n\
Are you sure you wish to SETUP the data acquisition system?\n\n\
SETUP will STOP data acquisition.\n\
Configure the VXI Control software and restore VXI parameter settings.\n\
This may take several minutes.\n\n\
If you decide to continue please wait for the SETUP sequence to complete and DO NOT touch the system while it is in progress."

set TestBuffer "0ff800c0  1ff80000 2ff80000  00d01234  20d02222  40d01111  90d04321 04d01234  24d02222  44d01111  94d04321 3ff80000"

proc checkinstallation  {} {
   global OS VERSION env

   global RDOUTEnable; if {$RDOUTEnable == 0} then {return 0}

   set ID [read-register EXEC.ID]
   set OS [first $ID]
   set version  [second $ID]
   set release [third $ID]

   if {$version != "D2VB"} then {
       set msg " WARNING!  Possible system installation or setup error! \n\
       Require Data Acquisition program $VERSION but discovered \n\
       OS $OS; Program $version; Release $release\n\
       Do you wish to continue?"
       if {[midas-confirmation $msg] == 0} then {return 1}
   }

   set env(RTOS) $OS
   return 0
}

proc Custom_Start {} {
    global env

   if {$env(MASTERCONSOLE) == "ALL"  || $env(MASTERCONSOLE) == [exec hostname]} then {
      inform rates start
      inform rates start-trig-rates
      inform rates start-sing-rates
#      inform rates start-tape-rates
      inform rates close-frame
 
      inform ecc start
      inform ecc close-frame
   }
   after 5000 auto-rates
}

proc do_system_console {} {
   global VXIEnable VMEEnable TSEnable

   set VXIEnable 1
   starttask vxicontrol
   starttask vxirestore

   set VMEEnable 1
   starttask vmecontrol
   starttask vmesaverestore

   set TSEnable 0 ;#     not controlled by default

   click-redisplay

   inform xm enableActions

      inform rates start

   inform rates set tapewarn 90
   inform rates set tapehalt 95

      inform rates start-trig-rates
      inform rates start-sing-rates
      inform rates start-tape-rates
      inform rates close-frame

      inform vre start
      inform vre close-frame

      inform ecc start
      inform ecc close-frame

}

proc do_slave_console {} {
   global VXIEnable VMEEnable TSEnable

   set VXIEnable 0
   set VMEEnable 0
   set TSEnable 0

   click-redisplay

   inform xm disableActions

      inform rates start

   inform rates set tapewarn disabled
   inform rates set tapehalt disabled

      inform rates stop-trig-rates
      inform rates stop-sing-rates
      inform rates stop-tape-rates
      inform rates close-frame

}

proc Make_Configuration_File {} {
  global env

    inform vxicontrol autostart
    inform vxicontrol "Make_Configuration_File"
    set-footer "Exogam configuration file written to $env(EXPERIMENTBASE)"
}

proc show_main_menu {} {
   global env
#    see if security option enabled and check permissions if it is
   if {$env(MIDASSECURITY) != "disabled"} then {
      if {![inform security validate "bf-menu,c"]} then {
#    user does not have permission to open bf menu
         set msg "Access Denied\n\nA password is required to access the main menu and you have not supplied it"
         set z [catch {midas_dialog [self frame-name].warningDialog "Attempted Security Violation" \
           "$msg" "warning" 0 "OK" "supply password"} m]
         if {$z != 0} then {
            puts stderr "midas-warning: [self frame-name].warningDialog: $m"
            catch {destroy [self frame-name].warningDialog}
         } else {
            if {$m == 1} then {inform security start}
         }
         return "Access Denied"
      }
   }
   send bf open-frame
}

proc exogam_state {} {
  global VXIEnable TSEnable RDOUTEnable
  global vxistate  tapestate readoutstate

#    if any one of the controlled subsystems is in the GO state then the
#    overall state is GO

  set state 0

  if {$VXIEnable == 1 && $vxistate == 1} then {set state 1}
  if {$RDOUTEnable == 1 && $readoutstate == 1} then {set state 1}
  if {$TSEnable == 1 && $tapestate == 2} then {set state 1}

  return $state
}

proc click-exogam {action} {click-da $action}

proc Reset_Electronics {} {
    global VXIEnable

    if {$VXIEnable == 1} then {
        click-vxi 2
        inform singles initialise-links
    }
}

proc Fixup_Electronics {} {loadsource VXIfixup}
proc Custom_PostGo     {} {loadsource VXIcheck}

proc update-rates {} {

    set rates [list 0 0 0 0 0 0 0 0]

    set-footer "Updating Rates"

    set names [inform rates inquire ScalerNames]
    set i 1
    foreach name $names {
      :modify RATE$i :label $name
      incr i
    }

    set rates [inform rates inquire triggerrates]
    foreach i {1 2 3 4 5 6 7 8} {
      :modify RATE$i :value [lindex $rates [expr $i - 1]]
    }

#    calculate deadtime -  warning this calculation is dependent on the trigger lca program

    set ftr1 [lindex $rates 0]
    set ftr2 [lindex $rates 1]
    set ftr3 [lindex $rates 2]
    set ftr4 [lindex $rates 3]
    set ft   [lindex $rates 4]

    set ftrtotal [expr $ftr1 + $ftr2 + $ftr3]
    set ftrlost  [expr $ftrtotal - $ft]

    if {$ftrtotal > 0} then {
       set deadtime [expr [expr $ftrlost * 100] / $ftrtotal]
    } else {
       set deadtime 0
    }
    :modify DEADTIME :value $deadtime
    :modify DTPC :label "[set deadtime]%"

    set names [inform rates get-singles-processors]
    set i 1
    foreach name $names {
      :modify SINGRATE$i :label [string toupper $name 0]
      incr i
    }

    set rates [inform rates inquire singlesrates]
    set i 1
    foreach name $names {
      if {$i < 3} then {:modify SINGRATE$i :value [lindex $rates [expr $i - 1]]}
      incr i
    }
 
    clear-footer
}

proc auto-rates {} {
   global autoperiod autocount
   after $autoperiod auto-rates
   incr autocount
   if {![:query_Mapped .]} then {return}
   if {$autocount < 10} then {update-rates} else {set autocount 0; click-redisplay}
}

proc delete_spectra {} {
   global SpectrumServer

   set z [catch {eg "(spectrum :path '/$SpectrumServer'  :name 'Stat' !delete)" } m]
   set z [catch {eg "(spectrum :path '/$SpectrumServer'  :name 'Rate' !delete)" } m]
}

proc create_spectra {} {
    global SpectrumServer HistMode EGreport Options

   set-footer "Creating spectra"

   set EGreport_SAVE $EGreport
   set EGreport -1

   set range $Options(Stat.channels)
   set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $range :name 'Stat' !new)" } m]

   set range $Options(Rate.channels)
   set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $range :name 'Rate' !new)" } m]
   
   map_needed

#     all other spectra created by the VXI software histogram interface

   set EGreport $EGreport_SAVE

   clear-footer
}

proc map_spectra {} {
   global EGreport RDOUThdwre HistMode SpectrumServer

   global RDOUTEnable; if {$RDOUTEnable == 0} then {return}

   Make_Xlate $SpectrumServer

   set-busy
   set-footer "Mapping spectra"

   set EGreport_SAVE $EGreport
   set EGreport -1

   set l [spectra-names]
   set num 0
   foreach item $l {
       incr num
       set name [second $item]
       if {"$name" == "Rate"} continue
       map_spectrum  [Do_XlateR $name] $name
       if {[expr ($num/25)*25] == $num} {set-footer "mapped spectrum $name"}
   }

   write-register $RDOUThdwre.Cspectramapped 1

   set EGreport $EGreport_SAVE

   set-footer "mapped [llength $l] spectra"
   clear-busy
}

proc map_needed {} {
    global RDOUThdwre
    write-register $RDOUThdwre.Cspectramapped 0
}

proc map_spectrum {specname alias} {

   global RDOUTEnable; if {$RDOUTEnable == 0} then {return}

   execute-command 10 "Mapping spectrum $specname ($alias)" "$specname $alias"
}

proc Custom_PreSetup {opt}  {

     if {$opt != 0} then {midas-report "Starting full SETUP. This will take several minutes. Please wait\n\
            See the Session Log for a progress report"}

#    setup hardware configuration information needed by the DA program

    set crates [inform vre inquire chanlist]
    if {$crates == "???"} then {set crates [list 15]}

    write-register NumVXICrates [llength $crates]

    set cratemap 0
    foreach c $crates {set cratemap [expr $cratemap + (1 << $c)]}
    write-register CrateMap $cratemap
}


proc Custom_PostSetup {opt}  {

     if {$opt != 0} then {inform vxicontrol Make_Configuration_File}
}

